/*
Programm: Aufg.05.1
Zweck:    Programm zur Modellierung von Kugel und Wuerfel als dreidimensionale
          Objekte. Zur Uebung von Vererbung und virtuellen Funktionen. 
Aufgabe:  Entspricht der Aufgabe 1 Blatt 5 der Uebungsaufgaben zur Vorlesung 
          "Programmierung 3" bei Prof. Henrich im WS 2002/2003.
Autor:    Mattias Ansorg <matthias@ansorgs.de>
Lizenz:   public domain
Datum:    2002-11-05
*/

#include <iostream>
using namespace std;

class RaumPunkt {
  private:
    float _x, _y, _z; //kartesische Raumkoordinaten des Punktes
  public:
    RaumPunkt();
    RaumPunkt(float x, float y, float z);
    void write();
};

class RaumObjekt {
  public:
    ~RaumObjekt();
    virtual float oberflaeche()=0;
    virtual void write()=0;
};

class RaumObjektKettenglied {
  private:
    RaumObjekt* _dieses;
    RaumObjektKettenglied* _naechstes;
  public:
    RaumObjektKettenglied(RaumObjekt *dieses);
    ~RaumObjektKettenglied();
    void set_naechstes(RaumObjektKettenglied *naechstes);
    RaumObjektKettenglied* get_naechstes();
    RaumObjekt* get_dieses();
};

class Kugel: public RaumObjekt {
  private:
   RaumPunkt _mpunkt;
   float _radius;
  public:
    Kugel();
    Kugel(RaumPunkt mpunkt, float radius);
    ~Kugel();
    float oberflaeche();
    void write();
};

class Wuerfel: public RaumObjekt {
  private:
    RaumPunkt _eckpunkt;
    float _kantenlaenge;
  public:
    Wuerfel();
    Wuerfel(RaumPunkt eckpunkt, float kantenlaenge);
    ~Wuerfel();
    float oberflaeche();
    void write();
};

RaumPunkt::RaumPunkt()
  : _x(0.0), _y(0.0), _z(0.0) {
}

RaumPunkt::RaumPunkt(float x, float y, float z)
  : _x(x), _y(y), _z(z) {
}

void RaumPunkt::write() {
  cout <<  "x:" << _x 
       << " y:" << _y
       << " z:" << _z
       << endl;
}

RaumObjekt::~RaumObjekt() {
  cout << "(debug: RaumObjekt::~RaumObjekt called)" << endl;
}

RaumObjektKettenglied::RaumObjektKettenglied(RaumObjekt *dieses) {
  _naechstes = NULL;
  _dieses = dieses;
  cout << "(debug: RaumObjektKettenglied::RaumObjektKettenglied called)" << endl;
}

RaumObjektKettenglied::~RaumObjektKettenglied() {
  delete _dieses;
  delete _naechstes;
  cout << "(debug: RaumObjektKettenglied::~RaumObjektKettenglied called)" << endl;
}

void RaumObjektKettenglied::set_naechstes(RaumObjektKettenglied *naechstes) {
  _naechstes = naechstes;
}

RaumObjektKettenglied* RaumObjektKettenglied::get_naechstes() {
  return _naechstes;
}

RaumObjekt* RaumObjektKettenglied::get_dieses() {
  return _dieses;
}

Kugel::Kugel()
  : _mpunkt(0.0, 0.0, 0.0), _radius(1) {
  cout << "(debug: Kugel::Kugel called)" << endl;
}

Kugel::Kugel(RaumPunkt mpunkt, float radius) 
  : _mpunkt(mpunkt), _radius(radius) {
  cout << "(debug: Kugel::Kugel called)" << endl;
}

Kugel::~Kugel() {
  cout << "(debug: Kugel::~Kugel called)" << endl;
}

float Kugel::oberflaeche() {
  return 1.0/3.0 * 3.141592654 * _radius * _radius;
}

void Kugel::write() {
  cout << endl << "Kugel:" << endl
       << "  Mittelpunkt: ";
  _mpunkt.write();
  cout << "  Radius: " << _radius << endl;
}

Wuerfel::Wuerfel()
  : _eckpunkt(0.0, 0.0, 0.0), _kantenlaenge(1) {
  cout << "Wuerfel::Wuerfel called" << endl;
}

Wuerfel::Wuerfel(RaumPunkt eckpunkt, float kantenlaenge)
  : _eckpunkt(eckpunkt), _kantenlaenge(kantenlaenge) {
  cout << "(debug: Wuerfel::Wuerfel called)" << endl;
}

Wuerfel::~Wuerfel() {
  cout << "(debug: Wuerfel::~Wuerfel called)" << endl;
}

void Wuerfel::write() {
  cout << endl << "Wrfel:" << endl
       << "  Eckpunkt: ";
  _eckpunkt.write();
  cout << "  Kantenlaenge: " << _kantenlaenge << endl;
}

float Wuerfel::oberflaeche() {
  return 6 * _kantenlaenge * _kantenlaenge;
}

int main () {
  char artObjekt, weiteresObjekt;
  RaumObjektKettenglied *erstesObjektPtr=0, *letztesObjektPtr=0, *neuesObjektPtr=0;
  bool ersteEingabe=true;
  
  do {
    //INV: erstesObjektPtr  zeigt auf erstes  Element der Liste
    //     letztesObjektPtr zeigt auf letztes Element der Liste
    cout << "Kugel oder Wrfel eingeben? (k/w) ";
    cin >> artObjekt;
    if (artObjekt == 'k' || artObjekt == 'K') {
      float x,y,z,r;
      cout << "  Kugel: Koordinaten des Mittelpunktes (x y z)? ";
      cin >> x >> y >> z;
      cout << "  Kugel: Radius (r)? ";
      cin >> r;
      neuesObjektPtr = new RaumObjektKettenglied( new Kugel( *(new RaumPunkt(x,y,z)),r ) );
      if (!ersteEingabe) //d.h. letztesObjektPtr != 0
	letztesObjektPtr->set_naechstes(neuesObjektPtr);
      letztesObjektPtr = neuesObjektPtr; 
    }
    else 
      if (artObjekt == 'w' || artObjekt == 'W') {
	float x,y,z,l;
	cout << "  Wuerfel: Koordinaten des linken unteren Eckpunktes (x y z)? ";
	cin >> x >> y >> z;
	cout << "  Wuerfel: Kantenlaenge (l)? ";
	cin >> l;
	neuesObjektPtr = new RaumObjektKettenglied( new Wuerfel( *(new RaumPunkt(x,y,z)), l) );
	if (!ersteEingabe) //d.h. letztesObjektPtr != 0
          letztesObjektPtr->set_naechstes(neuesObjektPtr);
	letztesObjektPtr = neuesObjektPtr;
      }
      else
	cout << "Fehler bei Eingabe der Objektart" << endl;
    if (ersteEingabe) {
      erstesObjektPtr = letztesObjektPtr;
      ersteEingabe = false;
    }
    cout << "Weiteres Objekt eingeben (j/n)? ";
    cin >> weiteresObjekt;
  } while (weiteresObjekt == 'j' || weiteresObjekt == 'J');

  //Daten und Oberflaeche der eingegebenen Objekte ausgeben
  for (RaumObjektKettenglied* iPtr = erstesObjektPtr; iPtr; iPtr = iPtr->get_naechstes()) {
    iPtr->get_dieses()->write();
    cout << "  Oberflche: " << iPtr->get_dieses()->oberflaeche() << endl << endl;
  }

  //verkettete Liste mit Destruktoraufruf rekursiv lschen 
  delete erstesObjektPtr;
}
