/*
Programm: Aufg.09.1
Zweck:    Programm zum Ueben der Ueberladung von Operatoren in verschiedenen Varianten
Aufgabe:  Entspricht der Aufgabe 1 Blatt 9 der Uebungsaufgaben zur Vorlesung
          "Programmierung 3" bei Prof. Henrich im WS 2002/2003.
Autor:    Mattias Ansorg <matthias@ansorgs.de>
Lizenz:   public domain
Datum:    2002-12-03
*/

#include <iostream>
using namespace std;

//----------------------------declarations

class Vector {
    friend Vector operator+(Vector vec1, Vector vec2);
    friend istream &operator>>(istream &in, Vector &vec);
    friend ostream &operator<<(ostream &out, Vector vec);
  private:
    int _dim;   //vector's dimension
    int *_data; //the data in the vector; use as a array _data[0] ... _data[_dim-1]
  public:
    Vector(int dimension, int init);
    Vector(const Vector &oldVec); //copy constructor
    ~Vector();
    int &operator[](int index);
    void operator()(int index, int value);
    int operator*(Vector vec2);
};

Vector operator+(Vector vec1, Vector vec2);

istream &operator>>(istream &in, Vector &vec);

ostream &operator<<(ostream &out, Vector vec);

//----------------------------definitions

Vector::Vector(int dimension, int init) : _dim(dimension) {
  _data = new int[_dim];
  for (int i=0; i<_dim; ++i)
    _data[i] = init; //initialize all vector components with the same value
  //cerr << "Vector:Vector(int,int): debug: _dim = " << _dim << endl;
}

Vector::Vector(const Vector &oldVec) : _dim(oldVec._dim) {
  _data = new int[_dim];
  for (int i=0; i<_dim; ++i)
    _data[i] = oldVec._data[i];
}

Vector::~Vector() {
  delete[] _data;
  _data = 0;
}

int &Vector::operator[](int index) {
  if ((index > _dim) || (index < 1)) //index out of range
    cerr << "(index out of range:)";
  else
    return _data[index-1];
}

void Vector::operator()(int index, int value) {
  if ((index > _dim) || (index < 1)) //index out of range
    cerr << "(index out of range:)";
  else
    _data[index-1] = value;
}

int Vector::operator*(Vector vec2) {
  int result=0;
  if (_dim != vec2._dim)
    cerr << "error: vectors differ in dimensions" << endl;
  else
    for (int i=0; i<_dim; ++i) {
      result += _data[i] * vec2._data[i];
    }
  return result;
}

Vector operator+(Vector vec1, Vector vec2) {
  int minDim = (vec1._dim < vec2._dim) ? vec1._dim : vec2._dim;
  int maxDim = (vec1._dim > vec2._dim) ? vec1._dim : vec2._dim;
  Vector result(maxDim, 0);

  for (int i=0; i<minDim; ++i) {
    result._data[i] = vec1._data[i] + vec2._data[i];
  }
  if (maxDim == vec1._dim)
    for (int i=minDim; i<maxDim; ++i)
      result._data[i] = vec1._data[i];
  else //maxDim == vec2._dim
    for (int i=minDim; i<maxDim; ++i)
      result._data[i] = vec2._data[i];
  return result;
}

//format to read a vector is component-wise, e.g. 1 3 4 78 is (1;3;4;78)
istream &operator>>(istream &in, Vector &vec) {
  for (int i=0; i<vec._dim; ++i) {
    in >> vec._data[i];
  }
  return in;
}

ostream &operator<<(ostream &out, Vector vec) {
  for (int i=0; i<vec._dim; ++i)
    out << vec._data[i] << " ";
  return out;
}

int main() {
  //initialize two vectors, test initialization
  Vector v1(5, 1);  //Vector with 5 components which are 1
  Vector v2(5, 2);  //Vector with 5 components which are 2
  cout << "initial values of vector 1: " << v1 << endl;  //should be 1 1 1 1 1
  cout << "initial values of vector 2: " << v2 << endl << endl;  //should be 2 2 2 2 2

  //test of vector input (thereby overwriting its values) and output
  cout << "please give new values for vector 1: ";
  cin >> v1;
  cout << "values of vector 1 are now, using cout: " << v1 << endl << endl;

  //test of operator[] for output
  cout << "values of vector 1 are now, using operator[]: " << endl;
  for (int i=0; i<=6; ++i) {
    cout << v1[i] << " "; //first and last should write "index out of range"
  }
  cout << endl << endl;

  //test of operator[] and operator() for input
  int v11, v12;
  cout << "please give new value for vector 1, 1st component (using operator[]): ";
  cin >> v11;
  v1[1] = v11;
  cout << "please give new value for vector 1, 2nd component (using operator()): ";
  cin >> v12;
  v1(2,v12);
  cout << "now, vector 1 is " << v1;

  //test of vector addition
  cout << endl << "vector 1 + vector 2 = " << (v1+v2) << endl;

  //test of dot product
  cout << endl << "vector 1 * vector 2 = " << (v1*v2) << endl;
}
